@@ -201,11 +201,12 @@ void usage(void)
 		"\t[-p ppm_error (default: 0)]\n"
 		"\t[-E enable_option (default: none)]\n"
 		"\t    use multiple -E to enable multiple options\n"
-		"\t    edge:   enable lower edge tuning\n"
-		"\t    dc:     enable dc blocking filter\n"
-		"\t    deemp:  enable de-emphasis filter\n"
-		"\t    direct: enable direct sampling\n"
-		"\t    offset: enable offset tuning\n"
+		"\t    edge:    enable lower edge tuning\n"
+		"\t    dc:      enable dc blocking filter\n"
+		"\t    deemp:   enable de-emphasis filter\n"
+		"\t    direct:  enable direct sampling 1 (usually I)\n"
+		"\t    direct2: enable direct sampling 2 (usually Q)\n"
+		"\t    offset:  enable offset tuning\n"
 		"\tfilename ('-' means stdout)\n"
 		"\t    omitting the filename also uses stdout\n\n"
 		"Experimental options:\n"
@@ -245,6 +246,7 @@ sighandler(int signum)
 #else
 static void sighandler(int signum)
 {
+	signal(SIGPIPE, SIG_IGN);
 	fprintf(stderr, "Signal caught, exiting!\n");
 	do_exit = 1;
 	rtlsdr_cancel_async(dongle.dev);
@@ -892,7 +894,7 @@ static void *controller_thread_fn(void *arg)
 	/* set up primary channel */
 	optimal_settings(s->freqs[0], demod.rate_in);
 	if (dongle.direct_sampling) {
-		verbose_direct_sampling(dongle.dev, 1);}
+		verbose_direct_sampling(dongle.dev, dongle.direct_sampling);}
 	if (dongle.offset_tuning) {
 		verbose_offset_tuning(dongle.dev);}
 
@@ -926,8 +928,21 @@ void frequency_range(struct controller_state *s, char *arg)
 	int i;
 	start = arg;
 	stop = strchr(start, ':') + 1;
+	if (stop == (char *)1) { // no stop or step given
+		s->freqs[s->freq_len] = (uint32_t) atofs(start);
+		s->freq_len++;
+		return;
+	}
 	stop[-1] = '\0';
 	step = strchr(stop, ':') + 1;
+	if (step == (char *)1) { // no step given
+		s->freqs[s->freq_len] = (uint32_t) atofs(start);
+		s->freq_len++;
+		s->freqs[s->freq_len] = (uint32_t) atofs(stop);
+		s->freq_len++;
+		stop[-1] = ':';
+		return;
+	}
 	step[-1] = '\0';
 	for(i=(int)atofs(start); i<=(int)atofs(stop); i+=(int)atofs(step))
 	{
@@ -1106,6 +1121,8 @@ int main(int argc, char **argv)
 				demod.deemph = 1;}
 			if (strcmp("direct",  optarg) == 0) {
 				dongle.direct_sampling = 1;}
+			if (strcmp("direct2", optarg) == 0) {
+				dongle.direct_sampling = 2;}
 			if (strcmp("offset",  optarg) == 0) {
 				dongle.offset_tuning = 1;}
 			break;
